//
// Created by sebastien lato on 2024-03-14.
//

#include "division.h"

/**
 * division - Effectue la division binaire en utilisant la méthode de restauration des restes partiels.
 * @dividende: le nombre à diviser (unsigned short).
 * @diviseur: le nombre par lequel diviser (unsigned short).
 * Retourne le résultat de la division où les 16 bits supérieurs sont le reste et les 16 bits inférieurs sont le quotient.
 */
unsigned long division(unsigned short dividende, unsigned short diviseur) {
    unsigned int quotient = 0;
    unsigned int reste = 0;
    int i;

    for (i = 15; i >= 0; i--) {
        reste = (reste << 1) | ((dividende >> i) & 1);
        if (reste >= diviseur) {
            reste -= diviseur;
            quotient |= (1U << i);
        }
    }
    // Empaquete le quotient et le reste dans un long non signé
    return ((unsigned long)reste << 16) | quotient;
}

/**
 * ecrireResultatDansFichier - Écrit le résultat d'une division dans un fichier texte.
 * @fichier: pointeur vers le fichier ouvert en écriture (FILE *).
 * @numeroDivision: le numéro de la division en cours (int).
 * @dividende: le dividende de la division (unsigned short).
 * @diviseur: le diviseur de la division (unsigned short).
 * @quotient: le quotient résultant de la division (unsigned short).
 * @reste: le reste résultant de la division (unsigned short).
 * Cette fonction n'a pas de valeur de retour.
 */
void ecrireResultatDansFichier(FILE* fichier, int numeroDivision, unsigned short dividende, unsigned short diviseur, unsigned short quotient, unsigned short reste) {
    if (fichier != NULL) {
        fprintf(fichier, "Division %d: %u / %u = %u reste %u\n", numeroDivision, dividende, diviseur, quotient, reste);
    }
}

