//
// Created by sebastien lato on 2024-03-14.
//

#include "menu.h"
#include "division.h"
#include <stdio.h>
#include <stdlib.h>

/**
 * effectuerDivisions - Gère la saisie utilisateur pour plusieurs divisions et écrit les résultats dans un fichier.
 * @fichier: Pointeur vers le fichier où les résultats seront écrits.
 *
 * Cette fonction invite l'utilisateur à entrer le nombre de divisions à effectuer,
 * puis pour chaque division, demande le dividende et le diviseur. Elle effectue ensuite
 * la division à l'aide de la fonction division et écrit le résultat dans le fichier spécifié.
 * La fonction s'assure que les entrées sont valides et dans les plages spécifiées.
 */
void effectuerDivisions(FILE *fichier) {
    unsigned long dividende, diviseur; // Utilisez long pour une vérification préalable
    unsigned long resultat;
    int nombreDivisions, i;

    printf("Entrez le nombre de divisions à effectuer : ");
    while (scanf("%d", &nombreDivisions) != 1 || nombreDivisions <= 0) {
        while(getchar() != '\n');
        printf("Nombre invalide. Veuillez entrer un entier positif : ");
    }

    for (i = 1; i <= nombreDivisions; i++) {
        printf("Division %d:\n", i);

        do {
            printf("Entrez le dividende (entre 0 et 65535) : ");
            scanf("%lu", &dividende);
            while(getchar() != '\n');
            if (dividende > 65535) {
                printf("Le dividende doit être entre 0 et 65535.\n");
            }
        } while (dividende > 65535);

        do {
            printf("Entrez le diviseur (entre 1 et 65535) : ");
            scanf("%lu", &diviseur);
            while(getchar() != '\n');
            if (diviseur == 0 || diviseur > 65535) {
                printf("Le diviseur doit être entre 1 et 65535.\n");
            }
        } while (diviseur == 0 || diviseur > 65535);

        resultat = division((unsigned short)dividende, (unsigned short)diviseur);
        ecrireResultatDansFichier(fichier, i, (unsigned short)dividende, (unsigned short)diviseur, (unsigned short)(resultat & 0xFFFF), (unsigned short)(resultat >> 16));
        fflush(fichier); // Force l'écriture des résultats dans le fichier
    }
}

/**
 * menu - Point d'entrée pour effectuer les opérations de division.
 *
 * Cette fonction ouvre le fichier `resultats.txt` et appelle effectuerDivisions pour
 * traiter les divisions. Une fois les divisions terminées, elle ferme le fichier et
 * informe l'utilisateur que les résultats ont été écrits.
 */
void menu() {
    FILE *fichier = fopen("../resultats.txt", "w");
    if (fichier == NULL) {
        perror("Erreur lors de l'ouverture du fichier");
        exit(EXIT_FAILURE);
    }

    effectuerDivisions(fichier);

    fclose(fichier);
    printf("Les résultats ont été écrits dans resultats.txt\n");
}
