/*
     _______________________________________
    |	Nom			|	Frdrick Prusse	|
    |	Code Perm.	|	PERF20358800		|
    |	Cours		|	SIF-1053 (TP2)  	|
    |	quipier	|	0					|
    |_______________|_______________________|


    Description: Ce programme ralise des divisions de nombres entiers positifs de 16 bits
    en utilisant uniquement des oprations d'addition, de soustraction et de dcalage.
    Les rsultats sont affichs dans la console et enregistrs dans un fichier 'reponse.txt'.
*/

#include <stdio.h>		// Oprations d'entre et de sortie
#include <stdlib.h>		// Fonctions standard
#include <limits.h>		// Contient les limites des types de donnes
#include <windows.h>	// Ncessaire pour les fonctions spcifiques  Windows


//	[main]
//	Titre: Programme Principal
// 
//	Description: Demande  l'utilisateur de saisir le nombre de divisions  raliser ainsi que les
//               oprandes pour chaque division. Effectue les divisions en appelant la fonction 'division'.
//               Affiche les rsultats et les enregistre dans un fichier 'reponse.txt'.
// 
//	INPUT:       - On utilise l'entre standard pour la saisie des donnes utilisateur.
// 
//	OUTPUT:      - On crit les rsultats dans la console et dans un fichier texte.


// Dclaration de la fonction division pour tre utilise dans main.
unsigned long division(unsigned short dividende, unsigned short diviseur);

int main() {
    SetConsoleOutputCP(1252); // Configure la console pour supporter les caractres accentus.

    // Dclaration des variables.
    int nbrDivisions = 0; // Nombre de divisions que l'utilisateur souhaite effectuer.
    FILE* fichier = NULL; // Pointeur vers le fichier o les rsultats seront enregistrs.
     errno_t err; // Variable pour stocker le code d'erreur potentiel lors de l'ouverture du fichier.

    // Boucle pour obtenir un nombre valide de divisions  effectuer.
    do {
        printf("Entrer le nombre de divisions  effectuer: ");
        scanf_s("%d", &nbrDivisions);
        while (getchar() != '\n'); // Nettoie le buffer pour les caractres rsiduels.

        // Validation: entier positif.
        if (nbrDivisions <= 0) {
            printf("Le nombre de divisions doit tre un entier positif.\n");
        }
    } while (nbrDivisions <= 0);

    // Tentative d'ouverture du fichier de rponse en criture.
    err = fopen_s(&fichier, "reponse.txt", "w");
    // Gestion d'erreur si le fichier ne peut pas tre ouvert.
    if (err != 0 || fichier == NULL) {
        perror("Erreur lors de l'ouverture du fichier");
        return EXIT_FAILURE;
    }

    // Boucle pour effectuer chaque division demande.
    for (int i = 0; i < nbrDivisions; i++) {

        // Demande  l'utilisateur d'entrer le dividende et le diviseur a/b.
        printf("-----------------------------------------------------\n");
        printf("Division %d (format a/b) : ", i + 1);

        // Variables temporaires pour stocker les entres.
        unsigned int dividende_tmp, diviseur_tmp;
        // Si scanf_s ne retourne pas 2, cela signifie qu'on n'a pas russi  lire deux valeurs, indiquant une erreur de saisie de l'utilisateur ou un dpassement.
        // On vrifie que le diviseur
        if (scanf_s("%u/%u", &dividende_tmp, &diviseur_tmp) != 2 || diviseur_tmp == 0 || dividende_tmp > USHRT_MAX || diviseur_tmp > USHRT_MAX) {
            printf("Saisie invalide, les valeurs doivent tre des entiers non signs de 16 bits et le diviseur ne doit pas tre zro.\n");
            while (getchar() != '\n'); // Nettoie le buffer d'entre
            i--; // Ritre la mme itration
            continue;
        }

        // Conversion des entres valides en unsigned short pour la division.
        unsigned short dividende = (unsigned short)dividende_tmp;
        unsigned short diviseur = (unsigned short)diviseur_tmp;

        // Appel de la fonction division et rcupration du rsultat.
        unsigned long resultat = division(dividende, diviseur);
       
        // Sparation du rsultat en quotient et reste.
        unsigned short quotient = (unsigned short)(resultat & 0xFFFF);
        unsigned short reste = (unsigned short)(resultat >> 16);

        // Affichage des rsultats de la division  la console.
        printf("%hu / %hu = %hu reste %hu\n", dividende, diviseur, quotient, reste);
        fprintf(fichier, "Division %d: %hu / %hu = %hu reste %hu\n", i + 1, dividende, diviseur, quotient, reste);
    }
    
    // Fermeture du fichier.
    fclose(fichier);
    printf("-----------------------------------------------------\n");
    printf("Les rsultats ont t enregistrs dans 'reponse.txt'.\n");
    
    // Fin du programme avec succs.
    return EXIT_SUCCESS;
}


//	[division]
//	Titre: Division Binaire par Restauration des Restes Partiels
// 
//	Description: Effectue la division de deux nombres entiers positifs de 16 bits en utilisant
//               uniquement les oprations de dcalage, d'addition et de soustraction.
// 
//	INPUT:      - dividende (unsigned short, valeur) : Le nombre  diviser, doit tre compris entre 0 et USHRT_MAX.
//              - diviseur (unsigned short, valeur) : Le nombre par lequel on divise, doit tre non nul et compris entre 1 et USHRT_MAX.
// 
//	OUTPUT:     - Retour (unsigned long) : Le reste est stock dans les 16 bits de poids fort et
//                le quotient dans les 16 bits de poids faible de la valeur retourne.

unsigned long division(unsigned short dividende, unsigned short diviseur) {
    
    // Initialisation des variables pour le reste et le quotient.
    unsigned long reste = 0;
    unsigned long quotient = 0;
    
    // Boucle sur chaque bit du dividende.
    for (int i = 0; i < 16; i++) {
        // Dcalage du reste et ajout du bit courant du dividende.
        reste = (reste << 1) | ((dividende >> (15 - i)) & 1);
        // Vrification si le reste actuel est suffisant pour tre divis par le diviseur.
        if (reste >= diviseur) {
        // Soustraction du diviseur du reste et ajout du bit au quotient.
            reste -= diviseur;
            quotient |= (1 << (15 - i));
        }
    }
    // Retourne le rsultat de la division : Reste dans la partie haute, quotient dans la partie basse.
    return (reste << 16) | quotient;
}